@@ -15,7 +15,8 @@
 #' @return a string that contain TextGrid. If argument write is \code{TRUE}, then no output.
 #'
 #' @examples
-#' create_subannotation(textgrid = example_textgrid, tier = 1, overwrite = FALSE)
+#' create_subannotation(system.file("extdata", "test.TextGrid", package = "phonfieldwork"),
+#' tier = 1, overwrite = FALSE)
 #'
 #' @export
 #'
@@ -37,21 +38,20 @@ create_subannotation <- function(textgrid,
   }
 
   df <- phonfieldwork::tier_to_df(tg, tier = tier)
-  df
 
   if(omit_blank){
-    df <- df[df$annotation != "",]
+    df <- df[df$content != "",]
   }
 
   lapply(1:nrow(df), function(i){
-    t <- seq(df$start[i], df$end[i], length.out = each*(n_of_annotations+1))
-    data.frame(start = t[-length(t)],
-               end = t[-1])
+    t <- seq(df$time_start[i], df$time_end[i], length.out = each*(n_of_annotations+1))
+    data.frame(time_start = t[-length(t)],
+               time_end = t[-1])
   }) ->
     l
 
-  final <- Reduce(rbind, l)
-  final <- cbind(id = 1:nrow(final), final, annotation = "")
+  final <- do.call(rbind, l)
+  final <- cbind(id = 1:nrow(final), final, content = "")
   phonfieldwork::df_to_tier(final,
                             textgrid = textgrid,
                             tier_name = new_tier_name,
